@@ -1,6 +1,6 @@
 package Net::OSCAR::Screenname;
 
-$VERSION = 0.25;
+$VERSION = 0.50;
 
 use strict;
 use vars qw($VERSION);
@@ -10,13 +10,14 @@ use Net::OSCAR::OldPerl;
 
 use overload
 	"cmp" => "compare",
-	'""' => "stringify";
+	'""' => "stringify",
+	"bool" => "boolify";
 
 sub new($$) {
-	return $_[1] if ref($_[0]);
+	return $_[1] if ref($_[0]) or UNIVERSAL::isa($_[1], "Net::OSCAR::Screenname");
 	my $class = ref($_[0]) || $_[0] || "Net::OSCAR::Screenname";
 	shift;
-	my $name = shift;
+	my $name = "$_[0]"; # Make doubleplus sure that name isn't one of us
 	my $self = \$name;
 	bless $self, $class;
 	return $self;
@@ -30,4 +31,10 @@ sub compare {
 
 sub stringify { my $self = shift; return $$self; }
 
+sub boolify {
+	my $self = shift;
+	return 0 if !defined($$self) or $$self eq "" or $$self eq "0";
+	return 1;
+}
+
 1;
